#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>

#define BUF_SIZE 1024

struct addrinfo *lookup_host(char *host_name, char *port_name) {
    struct addrinfo *server;
    struct addrinfo hints;
    memset(&hints, 0, sizeof(hints));
    hints.ai_family = AF_UNSPEC;
    hints.ai_socktype = SOCK_STREAM;

    int rv = getaddrinfo(host_name, port_name, &hints, &server);
    if (rv != 0) {
        printf("getaddrinfo failed: %s\n", gai_strerror(rv));
        return NULL;
    }
    return server;
}

void run_client(int sock_fd) {
    char buf[BUF_SIZE];
    while (fgets(buf, BUF_SIZE, stdin) != NULL) {
        size_t len = strlen(buf);
        if (write(sock_fd, buf, len) == -1) {
            perror("write");
            return;
        }

        memset(buf, 0, BUF_SIZE);
        size_t bytes_read = 0;
        while (bytes_read < len) {
            ssize_t n = read(sock_fd, buf + bytes_read, BUF_SIZE - bytes_read - 1);
            if (n == -1) {
                perror("read");
                return;
            } else if (n == 0) {
                // Server has closed connection
                return;
            }
            bytes_read += n;
        }
        printf("Server replied: %s\n", buf);
    }
}

int main(int argc, char **argv) {
    if (argc < 3) {
        printf("Usage: %s <host> <port>\n", argv[0]);
        return 1;
    }

    struct addrinfo *server = lookup_host(argv[1], argv[2]);
    if (server == NULL) {
        return 1;
    }
    int sock_fd = socket(server->ai_family, server->ai_socktype, server->ai_protocol);
    if (sock_fd == -1) {
        perror("socket");
        return 1;
    }
    if (connect(sock_fd, server->ai_addr, server->ai_addrlen) == -1) {
        perror("connect");
        return 1;
    }

    freeaddrinfo(server);

    run_client(sock_fd);
    close(sock_fd);
    freeaddrinfo(server);
    return 0;
}
